import ClassList from "../classes/classList.js";
import Style from "../classes/style.js";
import { Node, NodeType } from "../nodes/Node.js";
import { TextNode } from "../nodes/TextNode.js";
export class SVGElement extends Node {
    constructor(tagName) {
        super(NodeType.ELEMENT_NODE);
        this.attributes = {};
        this.childNodes = [];
        this.classList = new ClassList(this);
        this.style = new Style(this);
        this.tagName = tagName;
    }
    get children() {
        return this.childNodes.filter(node => node.nodeType === NodeType.ELEMENT_NODE);
    }
    get innerHTML() {
        let html = "";
        for (let childNode of this.childNodes) {
            if (childNode.nodeType === NodeType.TEXT_NODE || childNode.nodeType === NodeType.COMMENT_NODE || childNode.nodeType === NodeType.CDATA_SECTION_NODE) {
                html += childNode.nodeValue ?? "";
            }
            else if (childNode.nodeType === NodeType.ELEMENT_NODE) {
                html += childNode.outerHTML;
            }
        }
        return html;
    }
    set innerHTML(text) {
        this.childNodes = [new TextNode(text)];
    }
    get outerHTML() {
        const attributes = Object.keys(this.attributes).map(key => `${key}="${this.attributes[key]}"`).join(" ");
        const tagNameAndAttributes = `${[this.tagName, attributes].filter(value => value !== "").join(" ")}`;
        if (this.childNodes.length === 0) {
            return `<${tagNameAndAttributes} />`;
        }
        else {
            return `<${tagNameAndAttributes}>${this.innerHTML}</${this.tagName}>`;
        }
    }
    set innerText(text) {
        this.childNodes = [new TextNode(text)];
    }
    get innerText() {
        return this.childNodes.filter(node => node.nodeType === NodeType.TEXT_NODE)[0]?.nodeValue ?? "";
    }
    setAttribute(attributeName, value) {
        this.attributes[attributeName] = value;
    }
    getAttribute(attributeName) {
        return this.attributes[attributeName] ?? null;
    }
    removeAttribute(attributeName) {
        delete this.attributes[attributeName];
    }
    toggleAttribute(attributeName, force) {
        if (this.hasAttribute(attributeName)) {
            if (force === true) {
                return true;
            }
            this.removeAttribute(attributeName);
            return false;
        }
        else {
            if (force === false) {
                return false;
            }
            this.setAttribute(attributeName, "");
            return true;
        }
    }
    hasAttribute(attributeName) {
        return this.attributes[attributeName] !== undefined;
    }
    hasAttributes() {
        return Object.keys(this.attributes).length > 0;
    }
    get className() {
        return this.classList.value;
    }
    set className(className) {
        this.classList.value = className;
    }
    get id() {
        return this.getAttribute("id");
    }
    set id(id) {
        if (id !== null) {
            this.setAttribute("id", id);
        }
    }
    appendChild(node) {
        this.childNodes.push(node);
        return node;
    }
    get firstChild() {
        return this.children[0] ?? null;
    }
    get lastChild() {
        return this.children[this.children.length - 1] ?? null;
    }
    hasChildNodes() {
        return this.childNodes.length > 0;
    }
    removeChild(child) {
        for (let c = this.children.length - 1; c >= 0; c--) {
            if (this.children[c] === child) {
                this.children.splice(c, 1);
                return child;
            }
        }
        throw new Error("Uncaught NotFoundError: Failed to execute 'removeChild' on 'Node': The node to be removed is not a child of this node.");
    }
    getElementsByClassName(className) {
        const elements = [];
        for (let child of this.children) {
            if (child.classList.contains(className)) {
                elements.push(...[child, ...child.getElementsByClassName(className)]);
            }
        }
        return elements;
    }
    getElementsByTagName(tagName) {
        const elements = [];
        for (let child of this.children) {
            if (child.tagName === tagName) {
                elements.push(...[child, ...child.getElementsByTagName(tagName)]);
            }
        }
        return elements;
    }
    getElementById(id) {
        for (let child of this.children) {
            if (child.id === id) {
                return child;
            }
            else {
                return child.getElementById(id);
            }
        }
        return null;
    }
    tabIndex(index) {
        this.setAttribute("tabindex", index + "");
        return this;
    }
}
//# sourceMappingURL=SVGElement.js.map